/***************************************************************************
                          geowidget.cpp  -  description
                             -------------------
    begin                : Mon Feb 14 2000
    copyright            : (C) 2000 by Marc Bartsch
    email                : marc.bartsch@topmail.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "geowidget.h"

GeoWidget::GeoWidget( QWidget *parent, const char *name ) : QWidget( parent,name )
{
 	setBackgroundMode( NoBackground );

	intCanvasWidth = MetricsCenter::xRes;
  intCanvasHeight = MetricsCenter::yRes;

  leftPressed = false;
  startup = true;
	isKiosk = false;

  setMouseTracking( true );
	staticOverlayList.setAutoDelete( true );
	moveOverlayList.setAutoDelete( true );

	//	intCanvas shows grid and objects. gridOnlyCanvas
	//	only the grid and coordinates.
  intCanvas 		 = new QPixmap( intCanvasWidth, intCanvasHeight );
  gridOnlyCanvas = new QPixmap( intCanvasWidth, intCanvasHeight );
  tempMoveCanvas = new QPixmap( intCanvasWidth, intCanvasHeight );

	drawGrid();
}

GeoWidget::~GeoWidget()
{
}

void GeoWidget::paint( QRect *rect )
{
	QPoint pt = MetricsCenter::mapCanvasToWidget( rect->topLeft() );
	bitBlt( this, pt, intCanvas, *rect );
}

void 	GeoWidget::paint( QList <QRect> list )
{	
	//	And now loop through all QRects in list and copy those that can	be seen.
	QRect *actual;

	for ( actual = list.first(); actual; actual = list.next() )
	{
		paint( actual );
	}
}

void GeoWidget::paintEvent( QPaintEvent * )
{
	paintWholeWidget();
}

void GeoWidget::paintWholeWidget()
{
	// if startup ( or after kiosk mode ) the grid should to be centered
	if ( startup )
	{
		MetricsCenter::intCanvasLeftTopX = ( intCanvasWidth - width() ) / 2;
		MetricsCenter::intCanvasLeftTopY = ( intCanvasHeight - height()) / 2;
	}

	//	If right border has moved so that we have to move the groid to the right	
	if ( ( intCanvasWidth - MetricsCenter::intCanvasLeftTopX ) < width() )
	{
		MetricsCenter::intCanvasLeftTopX -= ( width() - ( intCanvasWidth - MetricsCenter::intCanvasLeftTopX ) );
	}

	if ( ( intCanvasHeight - MetricsCenter::intCanvasLeftTopY ) < height() )
	{
		MetricsCenter::intCanvasLeftTopY -= ( height() - ( intCanvasHeight - MetricsCenter::intCanvasLeftTopY ) );
	}

	if ( isKiosk )
	{
		bitBlt( this, 0, 0, intCanvas, 0, 0, width(), height() );
	} else {
		paint( new QRect( MetricsCenter::intCanvasLeftTopX, MetricsCenter::intCanvasLeftTopY, width(), height() ) );
	}
}

void GeoWidget::drawObjects( QList <GeoObjects> & objectList )
{
	bitBlt( intCanvas, 0, 0, gridOnlyCanvas );
	
	drawObjects( objectList, intCanvas );

	paintWholeWidget();
}

void GeoWidget::drawObjects( QList <GeoObjects> & objectList, QPixmap *canvas )
{
	int MaxZOrder = 0;

  QPainter *p = new QPainter;
	p->begin( canvas );

	GeoObjects *actual;
  QListIterator<GeoObjects> it( objectList );

	//	For every layer go through all objects
	for ( int y = 0; y <= MaxZOrder; y++ )
	{
		for ( actual = it.toFirst(); actual; actual = ++it )
		{
			if ( actual->getZOrder() > MaxZOrder )
			{
				MaxZOrder = actual->getZOrder();
			}
			
			if ( actual->getZOrder() == y )
			{
				actual->paint( p );
			}
  	}
	}

	p->end();
	delete p;
}

void GeoWidget::drawMovedObjects( QList <GeoObjects> & objectList )
{
	//	First, delete old objects.
	QRect *actual;

/*
	//	Copy static objects back to canvas.
  QListIterator<QRect> it1( staticOverlayList );

 	for ( actual = it1.toFirst(); actual; actual = ++it1 )
 	{
 		bitBlt( intCanvas, actual->topLeft(), tempMoveCanvas, *actual );
 	}
*/

	//	Delete objects, that moved last time.
  QListIterator<QRect> it2( moveOverlayList );

 	for ( actual = it2.toFirst(); actual; actual = ++it2 )
 	{
 		bitBlt( intCanvas, actual->topLeft(), tempMoveCanvas, *actual );
 	}

	//	Draw objects at new position
	drawObjects( objectList, intCanvas );

/*
  QPainter *p = new QPainter;
	p->begin( intCanvas );
	QRect *actually;
  QListIterator<QRect> it4( moveOverlayList );
 	for ( actually = it4.toFirst(); actually; actually = ++it4 )
 	{
 		p->drawRect( *actually );
		cout << "width: " << actually->width() << " height: " << actually->height() << endl;
 	}
  QListIterator<QRect> it5( staticOverlayList );
 	for ( actually = it5.toFirst(); actually; actually = ++it4 )
 	{
 		p->drawRect( *actually );
 	}
	p->end();
	delete p;
*/
  paint( moveOverlayList );

	//	Save moveOverlay for next move.
	GeoObjects *actual2;
	moveOverlayList.clear();
  QListIterator<GeoObjects> it3( objectList );

 	for ( actual2 = it3.toFirst(); actual2; actual2 = ++it3 )
 	{
		actual2->getObjectOverlay( moveOverlayList );
 	}

  paint( staticOverlayList );
  paint( moveOverlayList );
}

void GeoWidget::prepareMoving( 	QList <GeoObjects> & staticObjectList,
																QList <GeoObjects> & moveObjectList   )
{
	//	Fill tempMoveCanvas with grid Background.
	bitBlt( tempMoveCanvas,0, 0, gridOnlyCanvas );

	//	Save list of overlay rects of these static and move objects.
	staticOverlayList.clear();
	moveOverlayList.clear();

	GeoObjects *actual;
  QListIterator<GeoObjects> it1( staticObjectList );

 	for ( actual = it1.toFirst(); actual; actual = ++it1 )
 	{
		actual->getObjectOverlay( staticOverlayList );
 	}

  QListIterator<GeoObjects> it2( moveObjectList );

 	for ( actual = it2.toFirst(); actual; actual = ++it2 )
 	{
		actual->getObjectOverlay( moveOverlayList );
 	}

	//	Draw objects, that will definitely not be moved on the temp canvas.
	drawObjects( staticObjectList, tempMoveCanvas );
}

void GeoWidget::clear()
{
 	bitBlt( tempMoveCanvas, 0, 0, gridOnlyCanvas );
 	bitBlt( intCanvas, 0, 0, gridOnlyCanvas );
}

void GeoWidget::mouseMoveEvent ( QMouseEvent *ev )
{
	Coordinates coords( ev->pos() );
	MetricsCenter::mapWidgetToCanvas( coords );

	if ( leftPressed )
	{
		emit mouseMoveLeftButton( coords );
	}

	emit mouseMove( coords );
}

void GeoWidget::mousePressEvent ( QMouseEvent *ev )
{
	Coordinates coords( ev->pos() );

	if ( ev->button() == RightButton )
	{		
		emit mouseRightPressEvent( coords );
	}

	if ( ev->button() == LeftButton )
	{		
		leftPressed = true;	
  	MetricsCenter::extCanvasPosX = ev->x();
		MetricsCenter::extCanvasPosY = ev->y();
		MetricsCenter::mapWidgetToCanvas( coords );
		emit mouseLeftPressEvent( coords );
	}
}

void GeoWidget::mouseReleaseEvent ( QMouseEvent *ev )
{
	Coordinates coords( ev->pos() );
	MetricsCenter::mapWidgetToCanvas( coords );

	if ( ev->button() == LeftButton )
	{		
  	leftPressed = false;
		emit mouseLeftClickEvent( coords );
	}
}

void GeoWidget::focusOutEvent( QFocusEvent * )
{
	emit focusOut();
}

void GeoWidget::resizeEvent ( QResizeEvent * )
{
	if ( intCanvasWidth - intCanvasLeftTopX < width() )
	{
		intCanvasLeftTopX = intCanvasWidth - width() + 2;
	}

	if ( intCanvasHeight - intCanvasLeftTopY < height() )
	{
		intCanvasLeftTopY = intCanvasHeight - height() + 2;
	}
}

void GeoWidget::setStartup( bool start )
{
	startup = start;
}

void GeoWidget::dragEnterEvent( QDragEnterEvent* event )
{
	event->accept( QUriDrag::canDecode( event ) );
}

void GeoWidget::dropEvent( QDropEvent* event )
{
	QStrList files;

	if ( QUriDrag::decode( event, files ) )
	{
		emit loadDropFile( QUriDrag::uriToLocalFile( files.at(0) ) );
	}
}

void GeoWidget::keyPressEvent ( QKeyEvent *ev )
{
	ev->ignore();
}

void GeoWidget::drawGrid( QPainter *p )
{
	//	int realWidth =		MetricsCenter::getRealXRes();
	//	int realHeight =	MetricsCenter::getRealYRes();
	int left = 				MetricsCenter::getGridRect().left();
	int right = 			MetricsCenter::getGridRect().right();
	int top = 				MetricsCenter::getGridRect().top();
	int bottom = 			MetricsCenter::getGridRect().bottom();
	int xDistance = 	MetricsCenter::getXDist();
	int yDistance = 	MetricsCenter::getYDist();
	int xMid = 				MetricsCenter::getXMid();
	int yMid = 				MetricsCenter::getYMid();
	//  int realXMax = 		MetricsCenter::xRealMax;
  int realYMax = 		MetricsCenter::yRealMax;

	//	getGridConfig();

	// 	QPainter painter( paintDev );

/*
	if ( MetricsCenter::isPrinter )
	{
	 	painter.setPen( QPen( QColor( 255, 204, 0 ), 10, SolidLine ) );
		painter.drawRect( 0, 0, realWidth, realHeight );
	}
*/

 	int k;

 	// Draw grid lines
 	if (  MetricsCenter::grid )
 	{
 		p->setPen( QPen( lightGray, 0.5, DotLine ) );
 		for ( k = left; k <= right; k = k + xDistance )
 		{
 			p->drawLine( k, top, k, bottom );
 		}
 		for ( k = top; k <= bottom; k = k + yDistance )
 		{
 			p->drawLine( left, k, right, k );
 		}
 	}

 	// Draw axes
 	if (  MetricsCenter::axes )
 	{
 		p->setPen( QPen( darkGray, 1, SolidLine ) );
 		p->drawLine( left, yMid, right, yMid );
		p->drawLine( xMid, top, xMid, bottom );

		if ( MetricsCenter::isPrinter )
		{
	    QFont f( "helvetica", 10, QFont::Normal );
	 		p->setFont( f );
		}

 		// Draw Numbers
 		if (  MetricsCenter::numbers )
 		{
 			for ( k = left; k <= right; k = k + xDistance )
 			{
 				if ( k != xMid )
 				{
 					p->drawText( ( k - ( xDistance / 2 ) ) + 1 , yMid + 6, xDistance, yMid + 35, AlignHCenter|AlignTop,
 										QString().setNum( ( ( k - left ) / xDistance ) - ( MetricsCenter::xRealMax ) ) );
 				}
 			}
 			for ( k = top; k <= bottom; k = k + yDistance ){
 				if ( k != yMid ){
 					p->drawText( xMid + 2, k - 14, 24, 24, AlignCenter,
 														QString().setNum( realYMax -  ( ( k - top ) / yDistance )    ));
 				}
 			}
 		}

 		p->setPen( QPen( darkGray, 1, SolidLine ) );
 		p->setBrush( darkGray );
 		QPointArray a;
 		a.setPoints ( 3, right + 6, yMid - 3, right + 12 , yMid, right + 6, yMid + 3 );
 		p->drawPolygon( a, true );
 		p->drawLine( right, yMid, right + 5, yMid );
 		a.setPoints (3, xMid - 3, top - 6, xMid, top - 13, xMid + 3, top - 6 );
 		p->drawPolygon( a, true );
 	  p->drawLine( xMid, top, xMid, top - 6 );	
 	}
}

void GeoWidget::drawGrid()
{
	intCanvas->fill();
	gridOnlyCanvas->fill();
	tempMoveCanvas->fill();

	QPainter *p = new QPainter;
	p->begin( intCanvas );
	drawGrid( p );
	p->end();
	delete p;

	bitBlt( gridOnlyCanvas, 0, 0, intCanvas );
	bitBlt( tempMoveCanvas, 0, 0, intCanvas );
}
