/***************************************************************************
                          geowidget.h  -  description
                             -------------------
    begin                : Mon Feb 14 2000
    copyright            : (C) 2000 by Marc Bartsch
    email                : marc.bartsch@topmail.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GEOWIDGET_H
#define GEOWIDGET_H

#include <kapp.h>
#include <kconfig.h>

#include <qpoint.h>
#include <qstrlist.h>
#include <qlist.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qmultilinedit.h>
#include <qlabel.h>
#include <qdragobject.h>

#include "metricscenter.h"
#include "coordinates.h"
#include "geoobjects.h"

#include <iostream.h>

/**	
		This class represents the widget that shows the grid and all geometrical objects.
*/

class GeoWidget : public QWidget
{
		Q_OBJECT

	public:

  	/** Constructor */
  	GeoWidget( QWidget *parent=0, const char *name=0 );

  	/** Destructor */
  	~GeoWidget();

  	/** Widget used in kiosk mode? */
  	bool	isKiosk;

  	/** Internal Pixmap that contains coord grid and objects. This pixmap
  			is copied into the actual widget on the screen. */
    QPixmap	*intCanvas;

  	/** Internal Pixmap that contains only coord grid and NO objects. */
  	QPixmap	*gridOnlyCanvas;

  	/** Internal Pixmap for temporary reasons. */
  	QPixmap	*tempMoveCanvas;

  	/** Width of canvas. Is as big as screen resolution (for the kiosk mode) */
  	int 	intCanvasWidth;

  	/** Height of canvas. Is as big as screen resolution (for the kiosk mode) */
  	int 	intCanvasHeight;

  	/** Since the widget that shows the canvas on the screen is smaller than intCanvas,
        this is where the TopLeftX of the widget is positioned.	*/
    int 	intCanvasLeftTopX;

  	/**	Since the widget that shows the canvas on the screen is smaller than intCanvas,
        this is where the TopLeftY of the widget is positioned.	*/
  	int 	intCanvasLeftTopY;

  	/** Set canvas to startup */
    void	setStartup( bool start );

  	/** When canvas has to be painted, this method is called */
    void	paintEvent( QPaintEvent * );

  	/** This is where painting actually takes place (of rect) */
    void 	paint( QRect *rect );

  	/** This is where painting actually takes place (of rect) */
    void 	paint( QList <QRect> list );

  	/** Paints the whole canvas and not only rect */
    void 	paintWholeWidget();

  	/**	This method draws the objects contained in the list on the internal
  			canvas and updates the widget on screen.	*/
    void drawObjects( QList <GeoObjects> & objectList );

  	/**	This method draws the objects contained in the list on the internal
  			canvas and updates the widget on screen.	*/
		void drawMovedObjects( QList <GeoObjects> & objectList );

		/** This method does everything needed before moving. It creates a tempCanvas
				that contains the background and all objects that don't move. This is the
				background our moved objects will drawn upon. */
		void	prepareMoving( 	QList <GeoObjects> & staticObjectList,
													QList <GeoObjects> & moveObjectList   );

  	/**	Draws coordinates and everything on any paintDevice */
  	void drawGrid( QPainter *p );
		void drawGrid();

  	/**	Clears all objects and fills widget with background. */
		void clear();

  	/** All the events */
    void 	mouseMoveEvent( QMouseEvent *ev );
    void 	mousePressEvent( QMouseEvent *ev );
    void 	mouseReleaseEvent( QMouseEvent *ev );
  	void 	focusOutEvent( QFocusEvent * );
    void 	resizeEvent ( QResizeEvent * );
  	void	dragEnterEvent( QDragEnterEvent* event );
  	void	dropEvent( QDropEvent* event );
  	void	keyPressEvent ( QKeyEvent * );

  	int 	extCanvasPosX;
  	int 	extCanvasPosY;
    bool 	leftPressed;
  	bool  gridLines;
  	bool  numbers;
  	bool  axes;

	protected:

  	/**	This method draws the objects contained in the list on the canvas.	*/
    void drawObjects( QList <GeoObjects> & objectList, QPixmap *canvas );

    int 	drawingMode;
  	bool 	startup;
		QList <QRect> overlayList;
		QList <QRect> staticOverlayList;
		QList <QRect> moveOverlayList;

	signals:

    void mouseMoveLeftButton( Coordinates coords );
    void mouseMove( Coordinates coords );
    void mouseRelease( Coordinates coords );

  	void mouseRightPressEvent( Coordinates coords );
    void mouseLeftPressEvent( Coordinates coords );
    void mouseLeftClickEvent( Coordinates coords );

  	void loadDropFile( QString fileName );
  	void focusOut();
};

#endif
