/***************************************************************************
                          grid.cpp  -  description
                             -------------------
    begin                : Mon Feb 14 2000
    copyright            : (C) 2000 by Marc Bartsch
    email                : marc.bartsch@topmail.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "grid.h"

grid::grid(int x, int y) : QPixmap(x,y){

	gridWidth = x;												// Width of grid
	gridHeight= y;												// Height of grid
	distGridX = 30;												// Distance from one gridline to next Y
	distGridY = 30;  											// Distance from one gridline to next X
	minDistGridX = 30;										// Min Distance from one gridline to next Y
	minDistGridY = 30;  									// Min Distance from one gridline to next X
	leftTopX  = -gridWidth/2; 						// Coordinate left top
	leftTopY  = -gridHeight/2; 						// Coordinate right top
 	fill();
	// Draw grid for the whole background
//	drawWholeBackground();
}

grid::~grid(){
}

void grid::drawBackground(int X1, int Y1, int X2, int Y2)
{
	// Theses parameters represent the window that is going to be filled with
	// the background. The first time they should contain the whole grid.
  getGridConfig();

	x1 = X1;
	x2 = X2;
	y1 = Y1;
	y2 = Y2;

	p.begin(this);
	
	// Move Zeropoint (0,0) to center
	p.setWindow(leftTopX,leftTopY,gridWidth,gridHeight); 				

	// The rectangle that is to be drawn will probably
	// not lie on grid points. Therefore the parameters
	// x1,x2,y1,y2 have to be move to the next greater
	// rect that lies on grid points.
	if (x1 < 0){
		x1 = x1 - (x1 % distGridX) - distGridX;
	} else {
		x1 = x1 - (x1 % distGridX);
  }
	if (y1 < 0){
		y1 = y1 - (y1 % distGridY) - distGridY;
	} else {
		y1 = y1 - (y1 % distGridY);
  }
	if (x2 < 0){
		x2 = x2 - (x2 % distGridX);
	} else {
		x2 = x2 - (x2 % distGridX);
  }
	if (y2 < 0){
		y2 = y2 - (y2 % distGridY);
	} else {
		y2 = y2 - (y2 % distGridY) + distGridY;
  }
  x1 = x1 + distGridX;
	p.setPen( QPen(white,1, DotLine) );
	p.setBrush(white);
	p.drawRect(x1,y2,x2-x1,y2-y1);

  int k;

	// Draw grid lines
	if (gridLines){
		p.setPen( QPen(lightGray,1, DotLine) );
		for ( k = x1; k <= x2; k = k + distGridX){
			p.drawLine(k,y1,k,y2);
		}
		for ( k = y1; k <= y2; k = k + distGridY){
			p.drawLine(x1,k,x2,k);
	}
  }

	// Draw axes
	if (axes){
		p.setPen( QPen(darkGray,1, SolidLine) );
 		if (y1*y2 <= 0){
 			p.drawLine(x1,0,x2,0);
			// Draw Numbers
			if (numbers){
				for ( k = x1; k <= x2; k = k + distGridX){
					if (k != 0){
						p.drawText( k-(distGridX/2)+1, 6,distGridX,35 ,AlignHCenter|AlignTop,QString().setNum(k/distGridX));
					}
				}
			}
  	}

  	if (x1*x2 <= 0){
 			p.drawLine(0,y1,0,y2);
			// Draw Numbers
			if (numbers){
				for ( k = y1; k <= y2; k = k + distGridY){
					if (k != 0){
						p.drawText(5, k -8,25, 30,AlignHCenter|AlignTop,QString().setNum(-k/distGridY));
					}
				}
			}
  	}
  }

/*
draw little crosses
	int  i = x1;
	while (i <= x2)
	{
		int j = y1;																				// First grid marker on Y axis
  																										// Left coordinate of parameter
		while (j <= y2)
		{
			drawCross(i,j);
			j = j + distGridY;
		}
		i = i + distGridX;
  }	*/

  p.end();

}

void grid::drawWholeBackground()
{
  fill(white);
	drawBackground((-gridWidth/2) + distGridX ,(-gridHeight/2) + (2*distGridY) ,(gridWidth/2)-distGridX,(gridHeight/2)- (2*distGridY));

	// Add yellow frame and little arrows (top and right)
	p.begin(this);
	p.setPen( QPen(QColor(255,204,0),10, SolidLine) );
  p.drawRect(0,0,width(),height());

	// Move Zeropoint (0,0) to center
	p.setWindow(leftTopX,leftTopY,gridWidth,gridHeight); 				

  // Draw little arrows at the end of grid
	if (axes){
		p.setPen( QPen(darkGray,1, SolidLine) );
		p.setBrush(darkGray);
		QPointArray a;
		a.setPoints (3,x2 + 6,-3, x2+12 ,0,x2+6,3);
		p.drawPolygon(a, TRUE);
		p.drawLine(x2,0,x2+5,0);
		a.setPoints (3, -3, y1 - 6, 0, y1-13, 3, y1-6);
		p.drawPolygon(a, TRUE);
	  p.drawLine(0,y1,0,y1-6);	

	}
	p.end();
}

void grid::drawCross(int x, int y)
{
	p.drawLine(x-2,y,x+3,y);
	p.drawLine(x,y-2,x,y+3);
}

void grid::doubleGridDist()
{
	distGridX = distGridX * 2;												// Distance from one gridline to next Y
	distGridY = distGridY * 2;  											// Distance from one gridline to next X
}

void grid::halfGridDist()
{
	if (distGridX > minDistGridX)
	{
	distGridX = distGridX / 2;
	}
	if (distGridY > minDistGridY)
	{
	distGridY = distGridY / 2;
	}
}

void grid::getGridConfig()
{
	kapp->config()->setGroup("Grid");
	gridLines = kapp->config()->readBoolEntry("GridLines", TRUE);
	numbers	 	= kapp->config()->readBoolEntry("Numbers", TRUE);
	axes 			= kapp->config()->readBoolEntry("Axes", TRUE);
}

