/***************************************************************************
                          kgeoConstructor.cpp  -  description
                             -------------------
    begin                : Sun Oct 8 2000
    copyright            : (C) 2000 by Marc Bartsch
    email                : marc.bartsch@topmail.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//#include <kwin.h>

#include "kgeo.h"

KGeoApp::KGeoApp()
{
	startSplashScreen();

	//	Setup metrics object. The static metrics object contains method
  //  to calculate all sort of geometrical stuff.
	kapp->config()->setGroup( "Grid" );
	MetricsCenter::xMax = 					kapp->config()->readNumEntry( "HighestXValue", 11 );
	MetricsCenter::yMax = 					kapp->config()->readNumEntry( "HighestYValue", 7 );
	MetricsCenter::xRes = 					kapp->desktop()->width();
	MetricsCenter::yRes = 					kapp->desktop()->height();
	MetricsCenter::xDistanceRatio = kapp->config()->readNumEntry( "DistanceRatio", 5 );
	MetricsCenter::isPrinter = false;
	MetricsCenter::grid = 					kapp->config()->readBoolEntry( "GridLines", true );
	MetricsCenter::numbers = 				kapp->config()->readBoolEntry( "Numbers", true );
	MetricsCenter::axes = 					kapp->config()->readBoolEntry( "Axes", true );
	MetricsCenter::generateMetrics();

	// Get configuration object
  config = kapp->config();

	unsavedChanges = false;
	unnamedDrawing = true;

	//	Set Drawing mode to move Objects
	drawingMode = ID_drawingModeMovingObjects;

	//	Whether mouse is over object.
	isOverObject = false;

	//	Set the standard cursor for different modes.
  standardCursor = arrowCursor;

	//	List of all object for faster access.
	objectList.clear();

	//	Init current that object leads to the selected Object.
  currentObject = 0;

	//	Init new object to be drawn.
  newObject = 0;

	//	Create about window
	aboutWin = new AboutWindow( 0, 0, true, 0 );

	currentButtonID = 0;

	//	Create preferences window
	preferencesDialog = new PreferencesDialog( 0, 0, true, 0 );
	connect( preferencesDialog->buttonApply, SIGNAL( clicked() ), SLOT( savePreferences() ) );

	//	Create kiosk window
 	kiosk = new QDialog( 0, 0, false, WStyle_Customize | WStyle_NoBorder);
	kiosk->setGeometry( 0, 0, kapp->desktop()->width(), kapp->desktop()->height() );

  // call inits to invoke all other construction parts
  initMenuBar();
  initToolBar();
  initStatusBar();

	mainWidget = new MainWidget( this );

	canvas = new GeoWidget ( mainWidget->GeoContainer );
	canvas->setAcceptDrops( true );

  GeoWidgetLayout = new QHBoxLayout( mainWidget->GeoContainer );
  GeoWidgetLayout->setSpacing( 0 );
  GeoWidgetLayout->setMargin( 2 );
  GeoWidgetLayout->addWidget( canvas );

	setCentralWidget( mainWidget );

	//	So far there is only one context menu. Will be extended in future.
	kontextMenu = new QPopupMenu();
	kontextMenu->insertItem( i18n( "E&xit fullscreen mode" ), this, SLOT( endKioskMode() ) );

	//	Init currentFilename
	currentFilename = i18n( "No drawing" );
	printFilenameInWindowTitle();

	//	Connect those signal and slots that are necessary for the move mode.
	startMoveMode();
}

KGeoApp::~KGeoApp()
{
}

void KGeoApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR
  recentFilesMenu = new QPopupMenu();
  connect(recentFilesMenu, SIGNAL(aboutToShow()), SLOT(fillRecentMenu()));
  connect(recentFilesMenu, SIGNAL(activated(int)), SLOT(loadRecent(int)));
 	kapp->config()->setGroup("Recent");
	kapp->config()->readListEntry( "List", recentFiles );
	
  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  fileMenu = new QPopupMenu();
  fileMenu->insertItem(SmallIcon("filenew"), i18n("&New"), ID_FILE_NEW);
  fileMenu->insertItem(SmallIcon("fileopen"), i18n("&Open..."), ID_FILE_OPEN);
  fileMenu->insertItem(i18n("Open &recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);

  fileMenu->insertItem(i18n("&Close"), ID_FILE_CLOSE);
  fileMenu->insertSeparator();
  fileMenu->insertItem(SmallIcon("filesave") ,i18n("&Save"), ID_FILE_SAVE);
  fileMenu->insertItem(i18n("Save &As..."), ID_FILE_SAVE_AS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(SmallIcon("fileprint"), i18n("&Print..."), ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), ID_FILE_QUIT);

	fileMenu->setAccel( CTRL + Key_N, ID_FILE_NEW );
	fileMenu->setAccel( CTRL + Key_O, ID_FILE_OPEN );
	fileMenu->setAccel( CTRL + Key_C, ID_FILE_CLOSE );
	fileMenu->setAccel( CTRL + Key_S, ID_FILE_SAVE );
	fileMenu->setAccel( CTRL + Key_P, ID_FILE_PRINT );
	
  ///////////////////////////////////////////////////////////////////
  // menuBar entry editMenu
  editMenu = new QPopupMenu();
  editMenu->insertItem(SmallIcon("configure"), i18n("P&references"), ID_EDIT_PREFERENCES);
  editMenu->insertSeparator();
  editMenu->insertItem(SmallIcon("window_fullscreen"), i18n("&Fullscreen"), ID_EDIT_FULLSCREEN);

	editMenu->setAccel( CTRL + Key_R, ID_EDIT_PREFERENCES );

  ///////////////////////////////////////////////////////////////////
  // menuBar entry helpMenu
  helpMenu_ = helpMenu(  i18n("KGeo " VERSION "\n\n(c) 2001 by\n"
							"Marc Bartsch\nmarc.bartsch@web.de"));

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // insert your popup menus with the according menubar entries in the order
  // they will appear later from left to right
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), editMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), helpMenu_);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE MENU SLOTS WITH SIGNALS
  // for execution slots and statusbar messages
  connect( fileMenu, SIGNAL( activated( int ) ), SLOT( commandCallback( int ) ) );
  connect( editMenu, SIGNAL( activated( int ) ), SLOT( commandCallback( int ) ) );
  connect( editMenu, SIGNAL( highlighted( int ) ), SLOT( statusCallback( int ) ) );
}

void KGeoApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  toolBar()->insertButton( UserIcon("filenew"), ID_FILE_NEW, true, i18n("New File"));
  toolBar()->insertButton( UserIcon("fileopen"), ID_FILE_OPEN, true, i18n("Open File"));
  toolBar()->insertButton( UserIcon("filesave"), ID_FILE_SAVE, true, i18n("Save File"));
  toolBar()->insertButton( UserIcon("fileprint"), ID_FILE_PRINT, true, i18n("Print"));
  toolBar()->insertSeparator();
  toolBar()->insertLineSeparator();
  toolBar()->insertSeparator();
  toolBar()->insertButton( UserIcon("window_fullscreen"), ID_EDIT_FULLSCREEN, true, i18n("Fullscreen"));
  toolBar()->insertSeparator();
  toolBar()->insertLineSeparator();
  toolBar()->insertSeparator();
  toolBar()->insertButton( UserIcon("help"), ID_HELP_CONTENTS, SIGNAL(clicked()),
  				this, SLOT(appHelpActivated()), true,i18n("Help"));
  toolBar()->insertButton( UserIcon("exit"), ID_FILE_QUIT, SIGNAL(clicked()),
  				this, SLOT(slotFileQuit()), true,i18n("Quit"));

  connect(toolBar(), SIGNAL(clicked(int)), SLOT(commandCallback(int)));
  connect(toolBar(), SIGNAL(pressed(int)), SLOT(statusCallback(int)));

	KToolBar *point = new KToolBar( this, QMainWindow::Left, false );
	point->insertButton( UserIcon("point"), ID_buttonPoint, true, Str_buttonPoint );
	point->insertButton( UserIcon("pointxy"), ID_buttonPointxy, true, Str_buttonPointxy );
	point->insertButton( UserIcon("pointOnLine"), ID_buttonPointOnLine, true, Str_buttonPointOnLine  );
	point->insertButton( UserIcon("pointOfConc"), ID_buttonPointOfConc, true,   Str_buttonPointOfConc  );
	point->insertButton( UserIcon("bisection"), ID_buttonBisection, true,   Str_buttonBisection  );
	point->insertButton( UserIcon("mirrorPoint"), ID_buttonMirrorPoint, true,   Str_buttonMirrorPoint  );
	point->insertButton( UserIcon("rotation"), ID_buttonRotation, true,   Str_buttonRotation  );
	point->insertButton( UserIcon("move"), ID_buttonMove, true,   Str_buttonMove  );
	point->setFullSize( false );

	KToolBar *line = new KToolBar( this, QMainWindow::Left, false );
	line->insertButton( UserIcon("segment"), ID_buttonSegment, true,   Str_buttonSegment  );
	line->insertButton( UserIcon("line"), ID_buttonLine, true,   Str_buttonLine  );
	line->insertButton( UserIcon("triangle"), ID_buttonTriangle, true,   Str_buttonTriangle  );
	line->insertButton( UserIcon("vector"), ID_buttonVector, true,   Str_buttonVector  );
	line->insertButton( UserIcon("ray"), ID_buttonRay, true,   Str_buttonRay  );
	line->insertButton( UserIcon("parallel"), ID_buttonParallel, true,   Str_buttonParallel  );
	line->insertButton( UserIcon("perpendicular"), ID_buttonPerpendicular, true,   Str_buttonPerpendicular  );
	line->setFullSize( false );

	KToolBar *circle = new KToolBar( this, QMainWindow::Left, true );
	circle->insertButton( UserIcon("circle"), ID_buttonCircle, true,   Str_buttonCircle  );
	circle->insertButton( UserIcon("baseCircle"), ID_buttonBaseCircle, true,   Str_buttonBaseCircle  );
	circle->insertButton( UserIcon("arc"), ID_buttonArc, true,   Str_buttonArc  );
	circle->setFullSize( false );

	KToolBar *measure = new KToolBar( this, QMainWindow::Left, false );
	measure->insertButton( UserIcon("distance"), ID_buttonDistance, true,   Str_buttonDistance  );
	measure->insertButton( UserIcon("areaCircle"), ID_buttonArea, true,   Str_buttonArea  );
	measure->insertButton( UserIcon("angle"), ID_buttonAngle, true,   Str_buttonAngle  );
	measure->insertButton( UserIcon("slope"), ID_buttonSlope, true,   Str_buttonSlope  );
	measure->insertButton( UserIcon("circumference"), ID_buttonCircumference, true,   Str_buttonCircumference  );
	measure->setFullSize( false );

	KToolBar *tools = new KToolBar( this, QMainWindow::Left, false );
	tools->insertButton( UserIcon("attacher"), ID_buttonAttacher, true,   Str_buttonAttacher  );
	tools->insertButton( UserIcon("tracer"), ID_buttonTracer, true,   Str_buttonTracer  );
	tools->insertButton( UserIcon("eraser"), ID_buttonEraser, true,   Str_buttonEraser  );
	tools->insertButton( UserIcon("moveGrid"), ID_buttonMoveGrid, true,   Str_buttonMoveGrid  );
	tools->insertButton( UserIcon("text"), ID_buttonText, true,   Str_buttonText  );
	tools->setFullSize( false );

	KToolBar *colors = new KToolBar( this, QMainWindow::Right, false );
	colors->insertButton( UserIcon("black"), ID_buttonBlack, true,   Str_buttonBlack  );
	colors->insertButton( UserIcon("darkGray"), ID_buttonDarkGray, true,   Str_buttonDarkGray  );
	colors->insertButton( UserIcon("lightGray"), ID_buttonLightGray, true,   Str_buttonLightGray  );
	colors->insertButton( UserIcon("white"), ID_buttonWhite, true,   Str_buttonWhite  );
	colors->insertButton( UserIcon("blue"), ID_buttonBlue, true,   Str_buttonBlue  );
	colors->insertButton( UserIcon("red"), ID_buttonRed, true,   Str_buttonRed  );
	colors->insertButton( UserIcon("green"), ID_buttonGreen, true,   Str_buttonGreen  );
	colors->setFullSize( false );

	KToolBar *size = new KToolBar( this, QMainWindow::Right, false );
	size->insertButton( UserIcon("thinLine"), ID_buttonThinLine, true,   Str_buttonThinLine  );
	size->insertButton( UserIcon("middleLine"), ID_buttonMiddleLine, true,   Str_buttonMiddleLine  );
	size->insertButton( UserIcon("thickLine"), ID_buttonThickLine, true,   Str_buttonThickLine  );
	size->setFullSize( false );

  bg = new QButtonGroup( this );
  bg->hide();
  bg->setExclusive( true );

	for ( int i = ID_buttonPoint; i <= ID_buttonRotation; i++ )
	{
		point->setToggle ( i, true );
		bg->insert( point->getButton( i ), i );
  }
	for ( int i = ID_buttonSegment; i <= ID_buttonTriangle; i++ )
	{
		line->setToggle ( i, true );
		bg->insert( line->getButton( i ), i );
  }
	for ( int i = ID_buttonBaseCircle; i <= ID_buttonArc; i++ )
	{
		circle->setToggle ( i, true );
		bg->insert( circle->getButton( i ), i );
  }
	for ( int i = ID_buttonDistance; i <= ID_buttonCircumference; i++ )
	{
		measure->setToggle ( i, true );
		bg->insert( measure->getButton( i ), i );
  }
	for ( int i = ID_buttonBlack; i <= ID_buttonGreen; i++ )
	{
		colors->setToggle ( i, true );
		bg->insert( colors->getButton( i ), i );
  }
	for ( int i = ID_buttonThinLine; i <= ID_buttonThickLine; i++ )
	{
		size->setToggle ( i, true );
		bg->insert( size->getButton( i ), i );
  }
	for ( int i = ID_buttonEraser; i <= ID_buttonMoveGrid; i++ )
	{
		tools->setToggle ( i, true );
		bg->insert( tools->getButton( i ), i );
  }

  connect( bg, SIGNAL( clicked( int ) ), this, SLOT( toolBar2Clicked( int ) ) );
}

void KGeoApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  // STATUSBAR
  // TODO: add your own items you need for displaying current application status.
	statusBar()->setFixedHeight( 25 );
	statusBar()->insertFixedItem( i18n( " Welcome to KGeo                                                       " ),1 );
  statusBar()->insertItem( " 0 x 0                  ", 2, 2 );
	statusBar()->setItemAlignment ( 1, AlignLeft | AlignVCenter );
	statusBar()->setItemAlignment ( 2, AlignLeft | AlignVCenter );
}

void KGeoApp::startSplashScreen()
{	
	kapp->config()->setGroup( "Startup" );
	bool splashscreen = kapp->config()->readBoolEntry( "Splashscreen", true );

	if ( splashscreen )	
	{
		SplashScreen *splash = new SplashScreen();
		splash->exec();
		delete splash;
	}
}
