 /***************************************************************************
                          kgeoModeHandling.cpp  -  description
                             -------------------
    begin                : Sun Oct 8 2000
    copyright            : (C) 2000 by Marc Bartsch
    email                : marc.bartsch@topmail.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kgeo.h"

void KGeoApp::disconnectAll()
{
	// Disconnect from all important slots.
	canvas->disconnect();
	bg->disconnect();
}

void KGeoApp::startKioskMode()
{
	startMoveMode();
	connect( canvas, SIGNAL( mouseRightPressEvent( Coordinates ) ), this, SLOT( slotMouseRightPressEvent( Coordinates ) ) );

	canvas->isKiosk = true;
	canvas->reparent( kiosk, 0, QPoint( 0, 0 ), true );
	canvas->setGeometry( 0, 0, kapp->desktop()->width(), kapp->desktop()->height() );

	MetricsCenter::intCanvasLeftTopX = 0;
	MetricsCenter::intCanvasLeftTopY = 0;

	canvas->paintWholeWidget();
  canvas->setMouseTracking( true );

	kiosk->raise();
	kiosk->show();
}

void KGeoApp::endKioskMode()
{
	disconnectAll();

	canvas->isKiosk = false;
	canvas->reparent( mainWidget->GeoContainer, 0,QPoint( 2, 2 ),true );
  GeoWidgetLayout->addWidget( canvas );

	setCentralWidget( mainWidget );
  canvas->setMouseTracking( true );
	kiosk->hide();
	canvas->setStartup( true );
	canvas->paintWholeWidget();

	startMoveMode();
}

void KGeoApp::endMoveGridMode()
{
	startMoveMode();
}

void KGeoApp::startMoveMode ()
{
	disconnectAll();
	connect( canvas, SIGNAL( mouseMove( Coordinates ) ), this, SLOT( printCoordsInStatusline( Coordinates ) ) );
	connect( canvas, SIGNAL( mouseMove( Coordinates ) ), this, SLOT( changeMousePointer( Coordinates ) ) );
	connect( canvas, SIGNAL( mouseLeftPressEvent( Coordinates ) ), this, SLOT( selectPoint( Coordinates ) ) );
	connect( canvas, SIGNAL( loadDropFile( QString ) ), this, SLOT( loadDrawing( QString ) ) );
	connect( bg, 		 SIGNAL( clicked( int ) ), this, SLOT ( toolBar2Clicked( int ) ) );

	if ( canvas->isKiosk == true )
	{
		connect( canvas, SIGNAL( mouseRightPressEvent( Coordinates ) ), this, SLOT( slotMouseRightPressEvent( Coordinates ) ) );
	}

	//	Set standard cursor for move mode
	standardCursor = arrowCursor;
	canvas->setCursor( standardCursor );

	//	Set drawing mode to move
  drawingMode =  ID_drawingModeMovingObjects;

	//	Set buttons for move mode
	QButton *myButton = bg->find( currentButtonID );
	if ( myButton != 0 )
	{
		myButton->toggle();
		currentButtonID = -1;
	}

  statusBar()->changeItem( i18n( " Move objects mode" ), 1 );

	currentObject = 0;
	moveList.clear();
}

void KGeoApp::startDragMode()
{
	disconnectAll();
	connect( canvas, SIGNAL( mouseMove( Coordinates ) ), this, SLOT( printCoordsInStatusline( Coordinates ) ) );
	connect( canvas, SIGNAL( mouseMoveLeftButton( Coordinates ) ), this, SLOT( movePoint( Coordinates ) ) );
	connect( canvas, SIGNAL( mouseLeftClickEvent( Coordinates ) ), this, SLOT( releaseMouse( Coordinates ) ) );
}

void KGeoApp::startConstructionMode( QString statusLine )
{
	disconnectAll();
	connect( canvas, SIGNAL( mouseMove( Coordinates ) ), this, SLOT( printCoordsInStatusline( Coordinates ) ) );
	connect( canvas, SIGNAL( mouseLeftClickEvent( Coordinates ) ), this, SLOT( insertObject( Coordinates ) ) );
	connect( canvas, SIGNAL( mouseMove( Coordinates ) ), this, SLOT( changeMousePointer( Coordinates ) ) );
	connect( bg, 		 SIGNAL( clicked( int ) ), this, SLOT ( toolBar2Clicked( int ) ) );

	canvas->setCursor( arrowCursor );
  standardCursor = crossCursor;
  drawingMode = ID_drawingModeConstructing;

	statusBar()->changeItem( " " + statusLine, 1 );
}

void KGeoApp::startMoveGridMode()
{
	disconnectAll();
	connect( canvas, SIGNAL( mouseLeftClickEvent( QPoint* ) ), this, SLOT( endMoveGridMode() ) );
	connect( canvas, SIGNAL( mouseMove( QPoint* ) ), this, SLOT( moveGrid( QPoint* ) ) );
}
